@@ -1,5 +1,5 @@
 from abc import abstractmethod, ABC
-from typing import Tuple, List, Union, Dict, Any
+from typing import Tuple, List, Union, Dict, Any, Set
 import warnings
 import re
 
@@ -12,7 +12,7 @@ from optimade.server.config import CONFIG, SupportedBackend
 from optimade.server.exceptions import BadRequest, Forbidden
 from optimade.server.mappers import BaseResourceMapper
 from optimade.server.query_params import EntryListingQueryParams, SingleEntryQueryParams
-from optimade.server.warnings import FieldValueNotRecognized
+from optimade.server.warnings import FieldValueNotRecognized, UnknownProviderProperty
 
 
 def create_collection(
@@ -144,21 +144,43 @@ class EntryCollection(ABC):
                 )
 
         exclude_fields = self.all_fields - response_fields
+        include_fields = (
+            response_fields - self.resource_mapper.TOP_LEVEL_NON_ATTRIBUTES_FIELDS
+        )
+
+        bad_optimade_fields = set()
+        bad_provider_fields = set()
+        for field in include_fields:
+            if field not in self.resource_mapper.ALL_ATTRIBUTES:
+                if field.startswith("_"):
+                    if any(
+                        field.startswith(f"_{prefix}_")
+                        for prefix in self.resource_mapper.SUPPORTED_PREFIXES
+                    ):
+                        bad_provider_fields.add(field)
+                else:
+                    bad_optimade_fields.add(field)
+
+        if bad_provider_fields:
+            warnings.warn(
+                message=f"Unrecognised field(s) for this provider requested in `response_fields`: {bad_provider_fields}.",
+                category=UnknownProviderProperty,
+            )
+
+        if bad_optimade_fields:
+            raise BadRequest(
+                detail=f"Unrecognised OPTIMADE field(s) in requested `response_fields`: {bad_optimade_fields}."
+            )
 
         if results:
-            if isinstance(results, dict):
-                results = self.resource_cls(**self.resource_mapper.map_back(results))
-            else:
-                results = [
-                    self.resource_cls(**self.resource_mapper.map_back(doc))
-                    for doc in results
-                ]
+            results = self.resource_mapper.deserialize(results)
 
         return (
             results,
             data_returned,
             more_data_available,
             exclude_fields,
+            include_fields,
         )
 
     @abstractmethod
@@ -197,7 +219,7 @@ class EntryCollection(ABC):
 
         return fields
 
-    def get_attribute_fields(self) -> set:
+    def get_attribute_fields(self) -> Set[str]:
         """Get the set of attribute fields
 
         Return only the _first-level_ attribute fields from the schema of the resource class,
