@@ -1,9 +1,6 @@
-from typing import Union
-
 from fastapi import APIRouter, Depends, Request
 
 from optimade.models import (
-    ErrorResponse,
     ReferenceResource,
     ReferenceResponseMany,
     ReferenceResponseOne,
@@ -12,8 +9,8 @@ from optimade.server.config import CONFIG
 from optimade.server.entry_collections import create_collection
 from optimade.server.mappers import ReferenceMapper
 from optimade.server.query_params import EntryListingQueryParams, SingleEntryQueryParams
-
 from optimade.server.routers.utils import get_entries, get_single_entry
+from optimade.server.schemas import ERROR_RESPONSES
 
 
 router = APIRouter(redirect_slashes=True)
@@ -27,11 +24,14 @@ references_coll = create_collection(
 
 @router.get(
     "/references",
-    response_model=Union[ReferenceResponseMany, ErrorResponse],
+    response_model=ReferenceResponseMany,
     response_model_exclude_unset=True,
     tags=["References"],
+    responses=ERROR_RESPONSES,
 )
-def get_references(request: Request, params: EntryListingQueryParams = Depends()):
+def get_references(
+    request: Request, params: EntryListingQueryParams = Depends()
+) -> ReferenceResponseMany:
     return get_entries(
         collection=references_coll,
         response=ReferenceResponseMany,
@@ -42,13 +42,14 @@ def get_references(request: Request, params: EntryListingQueryParams = Depends()
 
 @router.get(
     "/references/{entry_id:path}",
-    response_model=Union[ReferenceResponseOne, ErrorResponse],
+    response_model=ReferenceResponseOne,
     response_model_exclude_unset=True,
     tags=["References"],
+    responses=ERROR_RESPONSES,
 )
 def get_single_reference(
     request: Request, entry_id: str, params: SingleEntryQueryParams = Depends()
-):
+) -> ReferenceResponseOne:
     return get_single_entry(
         collection=references_coll,
         entry_id=entry_id,
